# -*- coding: utf-8 -*-

# SPDX-FileCopyrightText: 2023 Jose D. Montoya
#
# SPDX-License-Identifier: MIT

import os
import sys
import datetime

sys.path.insert(0, os.path.abspath(".."))

try:
    # Inject mock modules so that we can build the
    # documentation without having the real stuff available
    from mock import Mock

    to_be_mocked = [
        "micropython",
        "machine",
    ]
    for module in to_be_mocked:
        sys.modules[module] = Mock()
        print("Mocked '{}' module".format(module))

    import micropython_mmr902
except ImportError:
    raise SystemExit("micropython_mmr902 has to be importable")

# -- General configuration ------------------------------------------------

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinx.ext.napoleon",
    "sphinx.ext.viewcode",
    "sphinx_immaterial",
]

autodoc_preserve_defaults = True

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "MicroPython": ("https://docs.micropython.org/en/latest/", None),
}

# autodoc_mock_imports = ["digitalio", "busio"]
autoclass_content = "both"
# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]
source_suffix = ".rst"
master_doc = "index"
# General information about the project.
project = "MicroPython mmr902 Library"
creation_year = "2023"
current_year = str(datetime.datetime.now().year)
year_duration = (
    current_year
    if current_year == creation_year
    else creation_year + " - " + current_year
)
copyright = year_duration + "Jose D. Montoya"
author = "Jose D. Montoya"

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = "1.0"
# The full version, including alpha/beta/rc tags.
release = "1.0"

language = "en"
autoclass_content = "both"
# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    ".env",
    "requirements.txt",
]

default_role = "any"
add_function_parentheses = True
pygments_style = "sphinx"
todo_include_todos = False
todo_emit_warnings = False
napoleon_numpy_docstring = False
html_baseurl = "https://micropython-mmr902.readthedocs.io/"
rst_prolog = """
.. role:: python(code)
   :language: python
   :class: highlight
.. default-literal-role:: python
"""
html_theme = "sphinx_immaterial"

html_theme_options = {
    "features": [
        "search.share",
    ],
    # Set the color and the accent color
    "palette": [
        {
            "media": "(prefers-color-scheme: light)",
            "scheme": "default",
            "primary": "purple",
            "accent": "light-blue",
            "toggle": {
                "icon": "material/lightbulb-outline",
                "name": "Switch to dark mode",
            },
        },
        {
            "media": "(prefers-color-scheme: dark)",
            "scheme": "slate",
            "primary": "purple",
            "accent": "light-blue",
            "toggle": {
                "icon": "material/lightbulb",
                "name": "Switch to light mode",
            },
        },
    ],
    # Set the repo location to get a badge with stats
    "repo_url": "https://github.com/jposada202020/MicroPython_MMR902/",
    "repo_name": "MicroPython MMR902",
    "social": [
        {
            "icon": "fontawesome/brands/github",
            "link": "https://github.com/jposada202020/MicroPython_MMR902",
        },
        {
            "icon": "fontawesome/brands/python",
            "link": "https://pypi.org/project/micropython-mmr902/",
        },
        {
            "name": "MicroPython Downloads",
            "icon": "octicons/download-24",
            "link": "https://micropython.org",
        },
    ],
}
html_favicon = "_static/favicon.ico"

# Output file base name for HTML help builder.
htmlhelp_basename = "MicroPython_Mmr920_Librarydoc"

sphinx_immaterial_custom_admonitions = [
    {
        "name": "warning",
        "color": (255, 66, 66),
        "icon": "octicons/alert-24",
        "override": True,
    },
    {
        "name": "note",
        "icon": "octicons/pencil-24",
        "override": True,
    },
    {
        "name": "seealso",
        "color": (255, 66, 252),
        "icon": "octicons/eye-24",
        "title": "See Also",
        "override": True,
    },
    {
        "name": "hint",
        "icon": "material/school",
        "override": True,
    },
    {
        "name": "tip",
        "icon": "material/school",
        "override": True,
    },
    {
        "name": "important",
        "icon": "material/school",
        "override": True,
    },
]
python_type_aliases = {
    "DigitalInOut": "digitalio.DigitalInOut",
}

object_description_options = [
    ("py:.*", dict(generate_synopses="first_sentence")),
]
# Set link name generated in the top bar.
html_title = "MicroPython MMR902"

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# These paths are either relative to html_static_path
# or fully qualified paths (eg. https://...)
html_css_files = ["extra_css.css"]


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    # 'preamble': '',
    # Latex figure (float) alignment
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (
        master_doc,
        "MicroPython_mmr902_Library.tex",
        "MicroPython mmr902 Library Documentation",
        author,
        "manual",
    ),
]

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (
        master_doc,
        "MicroPython_mmr902_Library",
        "MicroPython mmr902 Library Documentation",
        [author],
        1,
    ),
]

# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        master_doc,
        "MicroPython_mmr902_Library",
        "MicroPython mmr902 Library Documentation",
        author,
        "MicroPython_mmr902_Library",
        "One line description of project.",
        "Miscellaneous",
    ),
]



# SPDX-FileCopyrightText: Copyright (c) 2023 Jose D. Montoya
#
# SPDX-License-Identifier: MIT

import time
from machine import Pin, I2C
from micropython_mmr902 import mmr902

i2c = I2C(1, sda=Pin(2), scl=Pin(3))  # Correct I2C pins for RP2040
mmr = mmr902.MMR902(i2c)

mmr.operation_mode = mmr902.MODE1

while True:
    for operation_mode in mmr902.operation_mode_values:
        print("Current Operation mode setting: ", mmr.operation_mode)
        for _ in range(10):
            press = mmr.pressure
            print("Temperature: {:.2f}mmHg".format(press))
            print()
            time.sleep(0.5)
        mmr.operation_mode = operation_mode



# SPDX-FileCopyrightText: Copyright (c) 2023 Jose D. Montoya
#
# SPDX-License-Identifier: MIT

import time
from machine import Pin, I2C
from micropython_mmr902 import mmr902

i2c = I2C(1, sda=Pin(2), scl=Pin(3))  # Correct I2C pins for RP2040
mmr = mmr902.MMR902(i2c)

while True:
    print("Pressure: {}mmHg".format(mmr.pressure))
    print("Temperature: {}C".format(mmr.temperature))
    print()
    time.sleep(0.5)



# SPDX-FileCopyrightText: Copyright (c) 2023 Jose D. Montoya
#
# SPDX-License-Identifier: MIT
"""
`mmr902`
================================================================================

MicroPython Driver fro the Mitsumi MMR902 Micro Pressure Sensor


* Author(s): Jose D. Montoya


"""
import time
from micropython import const


__version__ = "0.1.0"
__repo__ = "https://github.com/jposada202020/MicroPython_MMR902.git"


MODE1 = const(0xA0)
MODE2 = const(0xA2)
MODE3 = const(0xA4)
MODE4 = const(0xA6)
operation_mode_values = (MODE1, MODE2, MODE3, MODE4)
_temp_waiting_values = {MODE1: 0.034, MODE2: 0.021, MODE3: 0.014, MODE4: 0.439}
_press_waiting_values = {MODE1: 0.016, MODE2: 0.08, MODE3: 0.004, MODE4: 0.256}


class MMR902:
    """Driver for the MMR902 Sensor connected over I2C.

    :param ~machine.I2C i2c: The I2C bus the MMR902 is connected to.
    :param int address: The I2C device address. Defaults to :const:`0x65`

    :raises RuntimeError: if the sensor is not found

    **Quickstart: Importing and using the device**

    Here is an example of using the :class:`MMR902` class.
    First you will need to import the libraries to use the sensor

    .. code-block:: python

        from machine import Pin, I2C
        from micropython_mmr902 import mmr902

    Once this is done you can define your `machine.I2C` object and define your sensor object

    .. code-block:: python

        i2c = I2C(1, sda=Pin(2), scl=Pin(3))
        mmr902 = mmr902.MMR902(i2c)

    Now you have access to the attributes

    .. code-block:: python

    """

    def __init__(self, i2c, address: int = 0x65) -> None:
        self._i2c = i2c
        self._address = address
        self._temp_waiting = None
        self._press_waiting = None
        self.operation_mode = MODE1

    @property
    def pressure(self) -> float:
        """
        Pressure in mmHg
        """
        data = bytearray(3)
        self._i2c.writeto(self._address, bytes([0xC0]))
        time.sleep(self._press_waiting)
        self._i2c.readfrom_into(self._address, data)
        result = data[0] << 16 | data[1] << 8 | data[2]

        return self._twos_comp(result, 24) / 1000

    @property
    def temperature(self) -> float:
        """
        Temperature in Celsius
        """
        data = bytearray(3)
        self._i2c.writeto(self._address, bytes([0xC2]))
        time.sleep(self._temp_waiting)
        self._i2c.readfrom_into(self._address, data)
        result = data[0] << 16 | data[1] << 8 | data[2]

        return self._twos_comp(result, 24) / 2**16.0

    @staticmethod
    def _twos_comp(val: int, bits: int) -> int:
        if val & (1 << (bits - 1)) != 0:
            return val - (1 << bits)
        return val

    @property
    def operation_mode(self) -> str:
        """
        Sensor operation_mode

        +--------------------------+------------------+
        | Mode                     | Value            |
        +==========================+==================+
        | :py:const:`mmr902.MODE1` | :py:const:`0XA0` |
        +--------------------------+------------------+
        | :py:const:`mmr902.MODE2` | :py:const:`0XA2` |
        +--------------------------+------------------+
        | :py:const:`mmr902.MODE3` | :py:const:`0XA4` |
        +--------------------------+------------------+
        | :py:const:`mmr902.MODE4` | :py:const:`0XA6` |
        +--------------------------+------------------+
        """
        values = {MODE1: "MODE1", MODE2: "MODE2", MODE3: "MODE3", MODE4: "MODE4"}
        return values[self._command]

    @operation_mode.setter
    def operation_mode(self, value: int) -> None:
        if value not in operation_mode_values:
            raise ValueError("Value must be a valid operation_mode setting")
        self._command = value
        self._temp_waiting = _temp_waiting_values[value]
        self._press_waiting = _press_waiting_values[value]
        self._i2c.writeto(self._address, bytes([self._command]))
